@@ -2,6 +2,7 @@
  *
  * Copyright © 2000 Keith Packard, member of The XFree86 Project, Inc.
  *             2005 Lars Knoll & Zack Rusin, Trolltech
+ *             2008 Aaron Plattner, NVIDIA Corporation
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -36,6 +37,11 @@
 #define Green(x) (((x) >> 8) & 0xff)
 #define Blue(x) ((x) & 0xff)
 
+#define Alpha64(x) ((x) >> 48)
+#define Red64(x) (((x) >> 32) & 0xffff)
+#define Green64(x) (((x) >> 16) & 0xffff)
+#define Blue64(x) ((x) & 0xffff)
+
 /*
  * Fetch from region strategies
  */
@@ -632,6 +638,30 @@ ACCESS(fbFetchTransformed)(bits_image_t * pict, int x, int y, int width,
     }
 }
 
+void
+ACCESS(fbFetchTransformed64)(bits_image_t * pict, int x, int y, int width,
+                             uint64_t *buffer, uint64_t *mask, uint32_t maskBits)
+{
+    // TODO: Don't lose precision for wide pictures!
+    uint32_t *mask8 = NULL;
+
+    // Contract the mask image, if one exists, so that the 32-bit fetch function
+    // can use it.
+    if (mask) {
+        mask8 = pixman_malloc_ab(width, sizeof(uint32_t));
+        pixman_contract(mask8, mask, width);
+    }
+
+    // Fetch the image into the first half of buffer.
+    ACCESS(fbFetchTransformed)(pict, x, y, width, (uint32_t*)buffer, mask8,
+                               maskBits);
+
+    // Expand from 32bpp to 64bpp in place.
+    pixman_expand(buffer, (uint32_t*)buffer, pict->format, width);
+
+    free(mask8);
+}
+
 #define SCANLINE_BUFFER_LENGTH 2048
 
 void
@@ -669,6 +699,45 @@ ACCESS(fbFetchExternalAlpha)(bits_image_t * pict, int x, int y, int width,
         free(alpha_buffer);
 }
 
+void
+ACCESS(fbFetchExternalAlpha64)(bits_image_t * pict, int x, int y, int width,
+                               uint64_t *buffer, uint64_t *mask,
+                               uint32_t maskBits)
+{
+    int i;
+    uint64_t _alpha_buffer[SCANLINE_BUFFER_LENGTH];
+    uint64_t *alpha_buffer = _alpha_buffer;
+    uint64_t maskBits64;
+
+    if (!pict->common.alpha_map) {
+        ACCESS(fbFetchTransformed64) (pict, x, y, width, buffer, mask, maskBits);
+	return;
+    }
+    if (width > SCANLINE_BUFFER_LENGTH)
+        alpha_buffer = (uint64_t *) pixman_malloc_ab (width, sizeof(uint64_t));
+
+    ACCESS(fbFetchTransformed64)(pict, x, y, width, buffer, mask, maskBits);
+    ACCESS(fbFetchTransformed64)((bits_image_t *)pict->common.alpha_map, x - pict->common.alpha_origin.x,
+                                 y - pict->common.alpha_origin.y, width,
+                                 alpha_buffer, mask, maskBits);
+
+    pixman_expand(&maskBits64, &maskBits, PIXMAN_a8r8g8b8, 1);
+
+    for (i = 0; i < width; ++i) {
+        if (!mask || mask[i] & maskBits64)
+	{
+	    int64_t a = alpha_buffer[i]>>48;
+	    *(buffer + i) = (a << 48)
+		| (div_65535(Red64(*(buffer + i)) * a) << 32)
+		| (div_65535(Green64(*(buffer + i)) * a) << 16)
+		| (div_65535(Blue64(*(buffer + i)) * a));
+	}
+    }
+
+    if (alpha_buffer != _alpha_buffer)
+        free(alpha_buffer);
+}
+
 void
 ACCESS(fbStoreExternalAlpha)(bits_image_t * pict, int x, int y, int width,
                              uint32_t *buffer)
@@ -710,3 +779,36 @@ ACCESS(fbStoreExternalAlpha)(bits_image_t * pict, int x, int y, int width,
 	   alpha_bits, buffer, ax - pict->common.alpha_origin.x, width, aindexed);
 }
 
+void
+ACCESS(fbStoreExternalAlpha64)(bits_image_t * pict, int x, int y, int width,
+                               uint64_t *buffer)
+{
+    uint32_t *bits, *alpha_bits;
+    int32_t stride, astride;
+    int ax, ay;
+    storeProc64 store;
+    storeProc64 astore;
+    const pixman_indexed_t * indexed = pict->indexed;
+    const pixman_indexed_t * aindexed;
+
+    store = ACCESS(pixman_storeProcForPicture64)(pict);
+    astore = ACCESS(pixman_storeProcForPicture64)(pict->common.alpha_map);
+    aindexed = pict->common.alpha_map->indexed;
+
+    ax = x;
+    ay = y;
+
+    bits = pict->bits;
+    stride = pict->rowstride;
+
+    alpha_bits = pict->common.alpha_map->bits;
+    astride = pict->common.alpha_map->rowstride;
+
+    bits       += y*stride;
+    alpha_bits += (ay - pict->common.alpha_origin.y)*astride;
+
+
+    store((pixman_image_t *)pict, bits, buffer, x, width, indexed);
+    astore((pixman_image_t *)pict->common.alpha_map,
+	   alpha_bits, buffer, ax - pict->common.alpha_origin.x, width, aindexed);
+}
